using Polly;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.
builder.Services.AddSingleton<IAsyncPolicy<HttpResponseMessage>>(
    Policy.HandleResult<HttpResponseMessage>(
        r => !r.IsSuccessStatusCode).RetryAsync(3));

var app = builder.Build();

app.MapGet("/Catalog/{id}", async (int id, IAsyncPolicy<HttpResponseMessage> policy) =>
{
    var cts = new CancellationTokenSource(3000);
    var cancellationToken = cts.Token;

    HttpClient httpClient = new HttpClient()
    {
        BaseAddress = new System.Uri(@"http://localhost:5000/")
    };
    string requestEndpoint = $"Inventory/{id}";

    HttpResponseMessage response;
    try
    {
        response = await policy.ExecuteAsync((ct) => httpClient.GetAsync(requestEndpoint, ct), cancellationToken);
    }
    catch (OperationCanceledException oce)
    {
        System.Console.WriteLine($"Request timed out, the cancellation token was triggered. {oce.Message}");
        return Results.Problem($"Request timed out, the cancellation token was triggered. {oce.Message}");
    }

    if (response.IsSuccessStatusCode)
    {
        int itemsInStock = await response.Content.ReadFromJsonAsync<int>();
        return Results.Ok(itemsInStock);
    }
    return Results.Problem("Something went wrong calling the inventory endpoint");
});

// Endpoint that fails 75% of the time, and is slow to respond.
int _requestCount = 0;

app.MapGet("/Inventory/{id}", async (int id) =>
{
    _requestCount++;

    await Task.Delay(TimeSpan.FromSeconds(1));
    if (_requestCount % 4 == 0) // only one of out four requests will succeed
    {
        System.Console.WriteLine($"{_requestCount} It worked!");
        return Results.Ok(15);
    }

    System.Console.WriteLine($"{_requestCount} Something went wrong");
    return Results.Problem("Something went wrong");
});

app.Run();